@@ -53,6 +53,27 @@ GList *chats = NULL;
 GtkWidget *all_chats = NULL;
 GtkWidget *chat_notebook = NULL;
 
+static char *ignored(struct conversation *b, char *who)
+{
+	GList *ignore = b->ignored;
+	while (ignore) {
+		char *ign = ignore->data;
+		if (!g_strcasecmp(who, ign))
+			return ign;
+		if (*ign == '+' && !g_strcasecmp(who, ign + 1))
+			return ign;
+		if (*ign == '@') {
+			ign++;
+			if (*ign == '+' && !g_strcasecmp(who, ign + 1))
+				return ign;
+			if (*ign != '+' && !g_strcasecmp(who, ign))
+				return ign;
+		}
+		ignore = ignore->next;
+	}
+	return NULL;
+}
+
 
 static void destroy_join_chat()
 {
@@ -120,7 +141,7 @@ static void rebuild_jc()
 		g_list_free(chatentries);
 	chatentries = NULL;
 
-	tmp = list = (*joinchatgc->prpl->chat_info)(joinchatgc);
+	tmp = list = joinchatgc->prpl->chat_info(joinchatgc);
 	while (list) {
 		GtkWidget *label;
 		GtkWidget *rowbox;
@@ -196,7 +217,7 @@ static void create_joinchat_menu(GtkWidget *box)
 			continue;
 		if (!joinchatgc)
 			joinchatgc = g;
-		g_snprintf(buf, sizeof buf, "%s (%s)", g->username, (*g->prpl->name)());
+		g_snprintf(buf, sizeof buf, "%s (%s)", g->username, g->prpl->name());
 		opt = gtk_menu_item_new_with_label(buf);
 		gtk_object_set_user_data(GTK_OBJECT(opt), g);
 		gtk_signal_connect(GTK_OBJECT(opt), "activate", GTK_SIGNAL_FUNC(joinchat_choose), g);
@@ -235,7 +256,7 @@ void join_chat()
 	}
 
 	if (!joinchat) {
-		joinchat = gtk_window_new(GTK_WINDOW_DIALOG);
+		GAIM_DIALOG(joinchat);
 		gtk_window_set_wmclass(GTK_WINDOW(joinchat), "joinchat", "Gaim");
 		gtk_window_set_policy(GTK_WINDOW(joinchat), FALSE, TRUE, TRUE);
 		gtk_widget_realize(joinchat);
@@ -365,7 +386,7 @@ void invite_callback(GtkWidget *w, struct conversation *b)
 	GtkWidget *frame;
 
 	if (!invite) {
-		invite = gtk_window_new(GTK_WINDOW_DIALOG);
+		GAIM_DIALOG(invite);
 		gtk_widget_realize(invite);
 
 		cancel = picture_button(invite, _("Cancel"), cancel_xpm);
@@ -442,7 +463,7 @@ void invite_callback(GtkWidget *w, struct conversation *b)
 
 void tab_complete(struct conversation *c)
 {
-	int pos = GTK_EDITABLE(c->entry)->current_pos;
+	int pos = GTK_OLD_EDITABLE(c->entry)->current_pos;
 	int start = pos;
 	int most_matched = -1;
 	char *entered, *partial = NULL;
@@ -584,7 +605,7 @@ void tab_complete(struct conversation *c)
 			g_free(tmp);
 			matches = g_list_remove(matches, matches->data);
 		}
-		write_to_conv(c, addthis, WFLAG_NOLOG, NULL, time(NULL));
+		write_to_conv(c, addthis, WFLAG_NOLOG, NULL, time(NULL), -1);
 		gtk_editable_insert_text(GTK_EDITABLE(c->entry), partial, strlen(partial), &start);
 		if (t == start) {
 			t = start + strlen(partial);
@@ -597,7 +618,7 @@ void tab_complete(struct conversation *c)
 	g_free(partial);
 }
 
-gboolean meify(char *message)
+gboolean meify(char *message, int len)
 {
 	/* read /me-ify : if the message (post-HTML) starts with /me, remove
 	 * the "/me " part of it (including that space) and return TRUE */
@@ -605,6 +626,8 @@ gboolean meify(char *message)
 	int inside_HTML = 0;	/* i really don't like descriptive names */
 	if (!c)
 		return FALSE;	/* um... this would be very bad if this happens */
+	if (len == -1)
+		len = strlen(message);
 	while (*c) {
 		if (inside_HTML) {
 			if (*c == '>')
@@ -616,12 +639,13 @@ gboolean meify(char *message)
 				break;
 		}
 		c++;		/* i really don't like c++ either */
+		len--;
 	}
 	/* k, so now we've gotten past all the HTML crap. */
 	if (!*c)
 		return FALSE;
 	if (!g_strncasecmp(c, "/me ", 4)) {
-		sprintf(c, "%s", c + 4);
+		memmove(c, c + 4, len - 3);
 		return TRUE;
 	} else
 		return FALSE;
@@ -669,7 +693,6 @@ static gboolean find_nick(struct gaim_connection *gc, char *message)
 
 void chat_write(struct conversation *b, char *who, int flag, char *message, time_t mtime)
 {
-	GList *ignore = b->ignored;
 	char *str;
 
 	if (!b->is_chat) {
@@ -677,26 +700,22 @@ void chat_write(struct conversation *b, char *who, int flag, char *message, time
 		return;
 	}
 
-	while (ignore) {
-		if (!g_strcasecmp(who, ignore->data))
-			return;
-		ignore = ignore->next;
-	}
-
+	if (ignored(b, who))
+		return;
 
 	if (!(flag & WFLAG_WHISPER)) {
 		str = g_strdup(normalize (who));
 		if (!g_strcasecmp(str, normalize(b->gc->username))) {
-			if (b->makesound && (sound_options & OPT_SOUND_CHAT_YOU_SAY))
-				play_sound(CHAT_YOU_SAY);
+			if (b->makesound)
+				play_sound(SND_CHAT_YOU_SAY);
 			flag |= WFLAG_SEND;
 		} else if (!g_strcasecmp(str, normalize(b->gc->displayname))) {
-			if (b->makesound && (sound_options & OPT_SOUND_CHAT_YOU_SAY))
-				play_sound(CHAT_YOU_SAY);
+			if (b->makesound)
+				play_sound(SND_CHAT_YOU_SAY);
 			flag |= WFLAG_SEND;
 		} else {
-			if (b->makesound && (sound_options & OPT_SOUND_CHAT_SAY))
-				play_sound(CHAT_SAY);
+			if (b->makesound)
+				play_sound(SND_CHAT_SAY);
 			flag |= WFLAG_RECV;
 		}
 		g_free(str);
@@ -705,7 +724,7 @@ void chat_write(struct conversation *b, char *who, int flag, char *message, time
 	if ((flag & WFLAG_RECV) && find_nick(b->gc, message))
 		flag |= WFLAG_NICK;
 
-	write_to_conv(b, message, flag, who, mtime);
+	write_to_conv(b, message, flag, who, mtime, -1);
 }
 
 
@@ -789,13 +808,40 @@ static void chat_press_ign(GtkWidget *obj, struct conversation *b)
 
 static void chat_press_info(GtkObject *obj, struct conversation *b)
 {
-	if (b->gc)
-		(*b->gc->prpl->get_info)(b->gc, gtk_object_get_user_data(obj));
+	if (b->gc) {
+		/*
+		 * If there are special needs for getting info on users in
+		 * buddy chat "rooms"...
+		 */
+		if(b->gc->prpl->get_cb_info != NULL) {
+			b->gc->prpl->get_cb_info(b->gc, b->id, gtk_object_get_user_data(obj));
+		} else {
+			b->gc->prpl->get_info(b->gc, gtk_object_get_user_data(obj));
+		}
+	}
+}
+
+
+static void chat_press_away(GtkObject *obj, struct conversation *b)
+{
+	if (b->gc) {
+		/*
+		 * May want to expand this to work similarly to chat_press_info?
+		 */
+		if(b->gc->prpl->get_cb_away != NULL) {
+			b->gc->prpl->get_cb_away(b->gc, b->id, gtk_object_get_user_data(obj));
+		}
+	}
 }
 
 static gint right_click_chat(GtkObject *obj, GdkEventButton *event, struct conversation *b)
 {
-	if (event->button == 3 && event->type == GDK_BUTTON_PRESS) {
+	if (event->button == 1 && event->type == GDK_2BUTTON_PRESS) {
+		struct conversation *c;
+		if ((c = find_conversation(gtk_object_get_user_data(obj))) == NULL)
+			c = new_conversation(gtk_object_get_user_data(obj));
+		set_convo_gc(c, b->gc);
+	} else if (event->button == 3 && event->type == GDK_BUTTON_PRESS) {
 		GtkWidget *menu;
 		GtkWidget *button;
 
@@ -807,7 +853,7 @@ static gint right_click_chat(GtkObject *obj, GdkEventButton *event, struct conve
 		gtk_menu_append(GTK_MENU(menu), button);
 		gtk_widget_show(button);
 
-		if (g_list_find_custom(b->ignored, gtk_object_get_user_data(obj), (GCompareFunc)strcmp))
+		if (ignored(b, gtk_object_get_user_data(obj)))
 			button = gtk_menu_item_new_with_label(_("Un-Ignore"));
 		else
 			button = gtk_menu_item_new_with_label(_("Ignore"));
@@ -825,53 +871,64 @@ static gint right_click_chat(GtkObject *obj, GdkEventButton *event, struct conve
 			gtk_widget_show(button);
 		}
 
+		if (b->gc && b->gc->prpl->get_cb_away) {
+			button = gtk_menu_item_new_with_label(_("Get Away Msg"));
+			gtk_signal_connect(GTK_OBJECT(button), "activate",
+					   GTK_SIGNAL_FUNC(chat_press_away), b);
+			gtk_object_set_user_data(GTK_OBJECT(button), gtk_object_get_user_data(obj));
+			gtk_menu_append(GTK_MENU(menu), button);
+			gtk_widget_show(button);
+		}
+
 		gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, event->button, event->time);
 		return TRUE;
 	}
 	return TRUE;
 }
 
+/*
+ * Common code for adding a chat buddy to the list
+ */
+static void add_chat_buddy_common(struct conversation *b, char *name, int pos)
+{
+        char tmp[BUF_LONG];
+        GtkWidget *list_item;
+
+        if (ignored(b, name)) {
+                g_snprintf(tmp, sizeof(tmp), "X %s", name);
+                list_item = gtk_list_item_new_with_label(tmp);
+        } else
+                list_item = gtk_list_item_new_with_label(name);
+
+        gtk_object_set_user_data(GTK_OBJECT(list_item), name);
+        gtk_signal_connect(GTK_OBJECT(list_item), "button_press_event",
+                           GTK_SIGNAL_FUNC(right_click_chat), b);
+        gtk_list_insert_items(GTK_LIST(b->list), g_list_append(NULL, list_item), pos);
+        gtk_widget_show(list_item);
+}
+
 void add_chat_buddy(struct conversation *b, char *buddy)
 {
 	char *name = g_strdup(buddy);
 	char tmp[BUF_LONG];
-	GtkWidget *list_item;
 	int pos;
-	GList *ignored;
 
 	plugin_event(event_chat_buddy_join, b->gc, (void *)b->id, name, 0);
 	b->in_room = g_list_insert_sorted(b->in_room, name, insertname);
 	pos = g_list_index(b->in_room, name);
 
-	ignored = b->ignored;
-	while (ignored) {
-		if (!g_strcasecmp(name, ignored->data))
-			 break;
-		ignored = ignored->next;
-	}
-
-	if (ignored) {
-		g_snprintf(tmp, sizeof(tmp), "X %s", name);
-		list_item = gtk_list_item_new_with_label(tmp);
-	} else
-		list_item = gtk_list_item_new_with_label(name);
-
-	gtk_object_set_user_data(GTK_OBJECT(list_item), name);
-	gtk_signal_connect(GTK_OBJECT(list_item), "button_press_event",
-			   GTK_SIGNAL_FUNC(right_click_chat), b);
-	gtk_list_insert_items(GTK_LIST(b->list), g_list_append(NULL, list_item), pos);
-	gtk_widget_show(list_item);
+	add_chat_buddy_common(b, name, pos);
 
 	g_snprintf(tmp, sizeof(tmp), _("%d %s in room"), g_list_length(b->in_room),
 		   g_list_length(b->in_room) == 1 ? "person" : "people");
 	gtk_label_set_text(GTK_LABEL(b->count), tmp);
 
-	if (b->makesound && (sound_options & OPT_SOUND_CHAT_JOIN))
-		play_sound(CHAT_JOIN);
+	if (b->makesound)
+		play_sound(SND_CHAT_JOIN);
 
 	if (chat_options & OPT_CHAT_LOGON) {
 		g_snprintf(tmp, sizeof(tmp), _("%s entered the room."), name);
-		write_to_conv(b, tmp, WFLAG_SYSTEM, NULL, time(NULL));
+		write_to_conv(b, tmp, WFLAG_SYSTEM, NULL, time(NULL), -1);
 	}
 }
 
@@ -882,9 +939,8 @@ void rename_chat_buddy(struct conversation *b, char *old, char *new)
 	GList *items = GTK_LIST(b->list)->children;
 
 	char *name = g_strdup(new);
-	GtkWidget *list_item;
+	char *ign;
 	int pos;
-	GList *ignored = b->ignored;
 
 	char tmp[BUF_LONG];
 
@@ -906,40 +962,44 @@ void rename_chat_buddy(struct conversation *b, char *old, char *new)
 		names = names->next;
 	}
 
-	if (!names)
+	if (!names) {
+		g_free(name);
 		return;
+	}
 
 	b->in_room = g_list_insert_sorted(b->in_room, name, insertname);
 	pos = g_list_index(b->in_room, name);
 
-	while (ignored) {
-		if (!g_strcasecmp(old, ignored->data))
-			break;
-		ignored = ignored->next;
+	ign = ignored(b, old);
+
+	if (ign) {
+		g_free(ign);
+		b->ignored = g_list_remove(b->ignored, ign);
+
+		/* we haven't added them yet. if we find them, don't add them again */
+		if (!ignored(b, new))
+			b->ignored = g_list_append(b->ignored, g_strdup(name));
+
+	} else {
+		if ((ign = ignored(b, new)) != NULL) {
+			/* if they weren't ignored and change to someone who is ignored,
+			 * assume it's someone else. that may seem a little backwards but
+			 * it's better when it *is* actually someone else. Sorry Sean. */
+			g_free(ign);
+			b->ignored = g_list_remove(b->ignored, ign);
+		}
 	}
 
-	if (ignored) {
-		b->ignored = g_list_remove(b->ignored, ignored->data);
-		b->ignored = g_list_append(b->ignored, name);
-		g_snprintf(tmp, sizeof(tmp), "X %s", name);
-		list_item = gtk_list_item_new_with_label(tmp);
-	} else
-		list_item = gtk_list_item_new_with_label(name);
-
-	gtk_object_set_user_data(GTK_OBJECT(list_item), name);
-	gtk_signal_connect(GTK_OBJECT(list_item), "button_press_event",
-			   GTK_SIGNAL_FUNC(right_click_chat), b);
-	gtk_list_insert_items(GTK_LIST(b->list), g_list_append(NULL, list_item), pos);
-	gtk_widget_show(list_item);
+	add_chat_buddy_common(b, name, pos);
 
 	if (chat_options & OPT_CHAT_LOGON) {
 		g_snprintf(tmp, sizeof(tmp), _("%s is now known as %s"), old, new);
-		write_to_conv(b, tmp, WFLAG_SYSTEM, NULL, time(NULL));
+		write_to_conv(b, tmp, WFLAG_SYSTEM, NULL, time(NULL), -1);
 	}
 }
 
 
-void remove_chat_buddy(struct conversation *b, char *buddy)
+void remove_chat_buddy(struct conversation *b, char *buddy, char *reason)
 {
 	GList *names = b->in_room;
 	GList *items = GTK_LIST(b->list)->children;
@@ -969,16 +1029,21 @@ void remove_chat_buddy(struct conversation *b, char *buddy)
 	if (!names)
 		return;
 
+	/* don't remove them from ignored in case they re-enter */
+
 	g_snprintf(tmp, sizeof(tmp), _("%d %s in room"), g_list_length(b->in_room),
 		   g_list_length(b->in_room) == 1 ? "person" : "people");
 	gtk_label_set_text(GTK_LABEL(b->count), tmp);
 
-	if (b->makesound && (sound_options & OPT_SOUND_CHAT_PART))
-		play_sound(CHAT_LEAVE);
+	if (b->makesound)
+		play_sound(SND_CHAT_LEAVE);
 
 	if (chat_options & OPT_CHAT_LOGON) {
-		g_snprintf(tmp, sizeof(tmp), _("%s left the room."), buddy);
-		write_to_conv(b, tmp, WFLAG_SYSTEM, NULL, time(NULL));
+		if (reason && *reason)
+			g_snprintf(tmp, sizeof(tmp), _("%s left the room (%s)."), buddy, reason);
+		else
+			g_snprintf(tmp, sizeof(tmp), _("%s left the room."), buddy);
+		write_to_conv(b, tmp, WFLAG_SYSTEM, NULL, time(NULL), -1);
 	}
 }
 
@@ -996,6 +1061,11 @@ void im_callback(GtkWidget *w, struct conversation *b)
 		return;
 	}
 
+	if (*name == '@')
+		name++;
+	if (*name == '+')
+		name++;
+
 	c = find_conversation(name);
 
 	if (c != NULL) {
@@ -1010,10 +1080,9 @@ void im_callback(GtkWidget *w, struct conversation *b)
 void ignore_callback(GtkWidget *w, struct conversation *b)
 {
 	char *name;
-	GList *i, *ignored;
+	GList *i;
+	char *ign;
 	int pos;
-	GtkWidget *list_item;
-	char tmp[80];
 
 	i = GTK_LIST(b->list)->selection;
 	if (i) {
@@ -1024,57 +1093,17 @@ void ignore_callback(GtkWidget *w, struct conversation *b)
 
 	pos = gtk_list_child_position(GTK_LIST(b->list), i->data);
 
-	ignored = b->ignored;
-	while (ignored) {
-		if (!g_strcasecmp(name, ignored->data))
-			 break;
-		ignored = ignored->next;
-	}
+	ign = ignored(b, name);
 
-	if (ignored) {
-		b->ignored = g_list_remove(b->ignored, ignored->data);
-		g_snprintf(tmp, sizeof tmp, "%s", name);
+	if (ign) {
+		g_free(ign);
+		b->ignored = g_list_remove(b->ignored, ign);
 	} else {
 		b->ignored = g_list_append(b->ignored, g_strdup(name));
-		g_snprintf(tmp, sizeof tmp, "X %s", name);
 	}
 
-	list_item = gtk_list_item_new_with_label(tmp);
-	gtk_object_set_user_data(GTK_OBJECT(list_item), name);
-	gtk_list_insert_items(GTK_LIST(b->list), g_list_append(NULL, list_item), pos);
 	gtk_widget_destroy(i->data);
-	gtk_widget_show(list_item);
-	gtk_signal_connect(GTK_OBJECT(list_item), "button_press_event",
-			   GTK_SIGNAL_FUNC(right_click_chat), b);
-}
-
-static gint delete_all_chats(GtkWidget *w, GdkEventAny *e, gpointer d)
-{
-	while (chats) {
-		struct conversation *c = chats->data;
-		close_callback(c->close, c);
-	}
-	return FALSE;
-}
-
-static void chat_switch(GtkNotebook *notebook, GtkWidget *page, gint page_num, gpointer data)
-{
-	GtkWidget *label = gtk_notebook_get_tab_label(GTK_NOTEBOOK(chat_notebook),
-						      gtk_notebook_get_nth_page(GTK_NOTEBOOK
-										(chat_notebook),
-										page_num));
-	GtkStyle *style;
-	struct conversation *b = g_list_nth_data(chats, page_num);
-	if (b && b->window && b->entry)
-		gtk_window_set_focus(GTK_WINDOW(b->window), b->entry);
-	if (!GTK_WIDGET_REALIZED(label))
-		return;
-	style = gtk_style_new();
-	gdk_font_unref(style->font);
-	style->font = gdk_font_ref(label->style->font);
-	gtk_widget_set_style(label, style);
-	gtk_style_unref(style);
-	b->unseen = FALSE;
+	add_chat_buddy_common(b, name, pos);
 }
 
 void show_new_buddy_chat(struct conversation *b)
@@ -1105,16 +1134,20 @@ void show_new_buddy_chat(struct conversation *b)
 	if (chat_options & OPT_CHAT_ONE_WINDOW) {
 		if (!all_chats) {
 			win = all_chats = b->window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+			if ((convo_options & OPT_CONVO_COMBINE) && (im_options & OPT_IM_ONE_WINDOW))
+				all_convos = all_chats;
 			gtk_window_set_wmclass(GTK_WINDOW(win), "buddy_chat", "Gaim");
-			gtk_window_set_policy(GTK_WINDOW(win), TRUE, TRUE, TRUE);
+			gtk_window_set_policy(GTK_WINDOW(win), TRUE, TRUE, FALSE);
 			gtk_container_border_width(GTK_CONTAINER(win), 0);
 			gtk_widget_realize(win);
 			aol_icon(win->window);
 			gtk_window_set_title(GTK_WINDOW(win), _("Gaim - Group Chats"));
 			gtk_signal_connect(GTK_OBJECT(win), "delete_event",
-					   GTK_SIGNAL_FUNC(delete_all_chats), NULL);
+					   GTK_SIGNAL_FUNC(delete_all_convo), NULL);
 
 			chat_notebook = gtk_notebook_new();
+			if ((convo_options & OPT_CONVO_COMBINE) && (im_options & OPT_IM_ONE_WINDOW))
+				convo_notebook = chat_notebook;
 			if (chat_options & OPT_CHAT_SIDE_TAB) {
 				if (chat_options & OPT_CHAT_BR_TAB) {
 					gtk_notebook_set_tab_pos(GTK_NOTEBOOK(chat_notebook),
@@ -1136,7 +1169,7 @@ void show_new_buddy_chat(struct conversation *b)
 			gtk_notebook_popup_enable(GTK_NOTEBOOK(chat_notebook));
 			gtk_container_add(GTK_CONTAINER(win), chat_notebook);
 			gtk_signal_connect(GTK_OBJECT(chat_notebook), "switch-page",
-					   GTK_SIGNAL_FUNC(chat_switch), NULL);
+					   GTK_SIGNAL_FUNC(convo_switch), NULL);
 			gtk_widget_show(chat_notebook);
 		} else
 			win = b->window = all_chats;
@@ -1267,7 +1300,7 @@ void show_new_buddy_chat(struct conversation *b)
 	if (convo_options & OPT_CONVO_CHECK_SPELLING)
 		gtkspell_attach(GTK_TEXT(chatentry));
 	gtk_box_pack_start(GTK_BOX(vbox), chatentry, TRUE, TRUE, 0);
-	gtk_widget_set_usize(chatentry, buddy_chat_size.width, buddy_chat_size.entry_height);
+	gtk_widget_set_usize(chatentry, buddy_chat_size.width, MAX(buddy_chat_size.entry_height, 25));
 	gtk_window_set_focus(GTK_WINDOW(win), chatentry);
 	gtk_widget_show(chatentry);
 
@@ -1304,8 +1337,6 @@ void show_new_buddy_chat(struct conversation *b)
 	b->log_dialog = NULL;
 	sprintf(b->fontface, "%s", fontface);
 	b->hasfont = 0;
-	b->fontsize = fontsize;
-	b->hassize = 0;
 	b->bgcol = bgcolor;
 	b->hasbg = 0;
 	b->fgcol = fgcolor;
@@ -1437,9 +1468,14 @@ void update_im_button_pix()
 void chat_tabize()
 {
 	int pos = 0;
+	char tmp[BUF_LONG];
 	/* evil, evil i tell you! evil! */
 	if (chat_options & OPT_CHAT_ONE_WINDOW) {
 		GList *x = chats;
+		if ((convo_options & OPT_CONVO_COMBINE) && (im_options & OPT_IM_ONE_WINDOW)) {
+			all_chats = all_convos;
+			chat_notebook = convo_notebook;
+		}
 		while (x) {
 			struct conversation *c = x->data;
 			GtkWidget *imhtml, *win;
@@ -1458,30 +1494,14 @@ void chat_tabize()
 			if (c->topic)
 				gtk_entry_set_text(GTK_ENTRY(c->topic_text), c->topic);
 
+			g_snprintf(tmp, sizeof(tmp), _("%d %s in room"), g_list_length(c->in_room),
+				   g_list_length(c->in_room) == 1 ? "person" : "people");
+			gtk_label_set_text(GTK_LABEL(c->count), tmp);
+
 			while (r) {
 				char *name = r->data;
-				GtkWidget *list_item;
-				GList *ignored = c->ignored;
-				char tmp[BUF_LONG];
 
-				while (ignored) {
-					if (!g_strcasecmp(name, ignored->data))
-						 break;
-					ignored = ignored->next;
-				}
-
-				if (ignored) {
-					g_snprintf(tmp, sizeof(tmp), "X %s", name);
-					list_item = gtk_list_item_new_with_label(tmp);
-				} else
-					list_item = gtk_list_item_new_with_label(name);
-
-				gtk_object_set_user_data(GTK_OBJECT(list_item), name);
-				gtk_signal_connect(GTK_OBJECT(list_item), "button_press_event",
-						   GTK_SIGNAL_FUNC(right_click_chat), c);
-				gtk_list_insert_items(GTK_LIST(c->list),
-						      g_list_append(NULL, list_item), pos);
-				gtk_widget_show(list_item);
+				add_chat_buddy_common(c, name, pos);
 
 				r = r->next;
 				pos++;
@@ -1507,30 +1527,14 @@ void chat_tabize()
 			if (c->topic)
 				gtk_entry_set_text(GTK_ENTRY(c->topic_text), c->topic);
 
+			g_snprintf(tmp, sizeof(tmp), _("%d %s in room"), g_list_length(c->in_room),
+				   g_list_length(c->in_room) == 1 ? "person" : "people");
+			gtk_label_set_text(GTK_LABEL(c->count), tmp);
+
 			while (r) {
 				char *name = r->data;
-				GtkWidget *list_item;
-				GList *ignored = c->ignored;
-				char tmp[BUF_LONG];
 
-				while (ignored) {
-					if (!g_strcasecmp(name, ignored->data))
-						 break;
-					ignored = ignored->next;
-				}
-
-				if (ignored) {
-					g_snprintf(tmp, sizeof(tmp), "X %s", name);
-					list_item = gtk_list_item_new_with_label(tmp);
-				} else
-					list_item = gtk_list_item_new_with_label(name);
-
-				gtk_object_set_user_data(GTK_OBJECT(list_item), name);
-				gtk_signal_connect(GTK_OBJECT(list_item), "button_press_event",
-						   GTK_SIGNAL_FUNC(right_click_chat), c);
-				gtk_list_insert_items(GTK_LIST(c->list),
-						      g_list_append(NULL, list_item), pos);
-				gtk_widget_show(list_item);
+				add_chat_buddy_common(c, name, pos);
 
 				r = r->next;
 				pos++;
@@ -1538,10 +1542,17 @@ void chat_tabize()
 
 			x = x->next;
 		}
-		if (all_chats)
+		chats = m;
+		if ((convo_options & OPT_CONVO_COMBINE) &&
+		    (im_options & OPT_IM_ONE_WINDOW) && conversations) {
+			while (m) {
+				gtk_notebook_remove_page(GTK_NOTEBOOK(chat_notebook),
+							 g_list_length(conversations));
+				m = m->next;
+			}
+		} else if (all_chats)
 			gtk_widget_destroy(all_chats);
 		all_chats = NULL;
 		chat_notebook = NULL;
-		chats = m;
 	}
 }
