@@ -773,7 +773,7 @@ static void gaimrc_read_options(FILE *f)
 	}
 
 	/* this is where we do bugs and compatibility stuff */
-	if (!(sound_options & (OPT_SOUND_BEEP | OPT_SOUND_ESD | OPT_SOUND_NORMAL | OPT_SOUND_NAS | OPT_SOUND_ARTSC)))
+	if (!(sound_options & (OPT_SOUND_BEEP | OPT_SOUND_ESD | OPT_SOUND_NORMAL | OPT_SOUND_NAS | OPT_SOUND_ARTSC | OPT_SOUND_CMD)))
 		sound_options |= OPT_SOUND_ESD | OPT_SOUND_NORMAL | OPT_SOUND_NAS | OPT_SOUND_ARTSC;
 
 	if (conv_size.width == 0 &&
@@ -1029,7 +1029,11 @@ static void set_defaults()
 	    OPT_SOUND_ARTSC |
 	    OPT_SOUND_ESD;
 
+#ifdef USE_SCREENSAVER
 	report_idle = IDLE_SCREENSAVER;
+#else
+	report_idle = IDLE_GAIM;
+#endif
 	web_browser = BROWSER_NETSCAPE;
 	g_snprintf(web_command, sizeof(web_command), "xterm -e lynx \"%%s\"");
 
