@@ -86,7 +86,7 @@ void remove_group(struct gaim_connection *gc, struct group *rem_g)
 
 	gc->groups = g_slist_remove(gc->groups, delg);
 
-	serv_remove_buddies(gc, tmp);
+	serv_remove_buddies(gc, tmp, rem_g->name);
 	while (tmp) {
 		g_free(tmp->data);
 		tmp = g_list_remove(tmp, tmp->data);
@@ -106,6 +106,9 @@ struct buddy *add_buddy(struct gaim_connection *gc, char *group, char *buddy, ch
 	struct group *g;
 	char *good;
 
+	if (!g_slist_find(connections, gc))
+		return NULL;
+
 	if ((b = find_buddy(gc, buddy)) != NULL)
 		return b;
 
@@ -123,7 +126,7 @@ struct buddy *add_buddy(struct gaim_connection *gc, char *group, char *buddy, ch
 	b->present = 0;
 
 	if (gc->prpl->normalize)
-		good = (*gc->prpl->normalize)(buddy);
+		good = gc->prpl->normalize(buddy);
 	else
 		good = buddy;
 
@@ -145,6 +148,8 @@ struct group *add_group(struct gaim_connection *gc, char *group)
 	struct group *g = find_group(gc, group);
 	if (g)
 		return g;
+	if (!g_slist_find(connections, gc))
+		return NULL;
 	g = (struct group *)g_new0(struct group, 1);
 	if (!g)
 		return NULL;
@@ -170,6 +175,8 @@ struct group *find_group(struct gaim_connection *gc, char *group)
 
 	strcpy(grpname, normalize (group));
 	if (gc) {
+		if (!g_slist_find(connections, gc))
+			return NULL;
 		grp = gc->groups;
 		while (grp) {
 			g = (struct group *)grp->data;
@@ -216,7 +223,7 @@ struct group *find_group_by_buddy(struct gaim_connection *gc, char *who)
 			norm = gc->prpl->normalize;
 		else
 			norm = normalize;
-		whoname = g_strdup((*norm)(who));
+		whoname = g_strdup(norm(who));
 		grp = gc->groups;
 		while (grp) {
 			g = (struct group *)grp->data;
@@ -224,7 +231,7 @@ struct group *find_group_by_buddy(struct gaim_connection *gc, char *who)
 			mem = g->members;
 			while (mem) {
 				b = (struct buddy *)mem->data;
-				if (!strcmp((*norm)(b->name), whoname)) {
+				if (!strcmp(norm(b->name), whoname)) {
 					g_free(whoname);
 					return g;
 				}
@@ -243,7 +250,7 @@ struct group *find_group_by_buddy(struct gaim_connection *gc, char *who)
 				norm = z->prpl->normalize;
 			else
 				norm = normalize;
-			whoname = g_strdup((*norm)(who));
+			whoname = g_strdup(norm(who));
 			grp = z->groups;
 			while (grp) {
 				g = (struct group *)grp->data;
@@ -251,7 +258,7 @@ struct group *find_group_by_buddy(struct gaim_connection *gc, char *who)
 				mem = g->members;
 				while (mem) {
 					b = (struct buddy *)mem->data;
-					if (!strcmp((*norm)(b->name), whoname)) {
+					if (!strcmp(norm(b->name), whoname)) {
 						g_free(whoname);
 						return g;
 					}
@@ -278,11 +285,13 @@ struct buddy *find_buddy(struct gaim_connection *gc, char *who)
 	char *(*norm)(const char *);
 
 	if (gc) {
+		if (!g_slist_find(connections, gc))
+			return NULL;
 		if (gc->prpl->normalize)
 			norm = gc->prpl->normalize;
 		else
 			norm = normalize;
-		whoname = g_strdup((*norm)(who));
+		whoname = g_strdup(norm(who));
 		grp = gc->groups;
 		while (grp) {
 			g = (struct group *)grp->data;
@@ -290,7 +299,7 @@ struct buddy *find_buddy(struct gaim_connection *gc, char *who)
 			mem = g->members;
 			while (mem) {
 				b = (struct buddy *)mem->data;
-				if (!strcmp((*norm)(b->name), whoname)) {
+				if (!strcmp(norm(b->name), whoname)) {
 					g_free(whoname);
 					return b;
 				}
@@ -308,7 +317,7 @@ struct buddy *find_buddy(struct gaim_connection *gc, char *who)
 				norm = z->prpl->normalize;
 			else
 				norm = normalize;
-			whoname = g_strdup((*norm)(who));
+			whoname = g_strdup(norm(who));
 			grp = z->groups;
 			while (grp) {
 				g = (struct group *)grp->data;
@@ -316,7 +325,7 @@ struct buddy *find_buddy(struct gaim_connection *gc, char *who)
 				mem = g->members;
 				while (mem) {
 					b = (struct buddy *)mem->data;
-					if (!strcmp((*norm)(b->name), whoname)) {
+					if (!strcmp(norm(b->name), whoname)) {
 						g_free(whoname);
 						return b;
 					}
@@ -331,7 +340,7 @@ struct buddy *find_buddy(struct gaim_connection *gc, char *who)
 	}
 }
 
-void parse_toc_buddy_list(struct gaim_connection *gc, char *config, int from_do_import)
+void parse_toc_buddy_list(struct gaim_connection *gc, char *config)
 {
 	char *c;
 	char current[256];
@@ -340,37 +349,44 @@ void parse_toc_buddy_list(struct gaim_connection *gc, char *config, int from_do_
 	int how_many = 0;
 
 	bud = NULL;
-
+    
 	if (config != NULL) {
-
+    
 		/* skip "CONFIG:" (if it exists) */
 		c = strncmp(config + 6 /* sizeof(struct sflap_hdr) */ , "CONFIG:", strlen("CONFIG:")) ?
-		    strtok(config, "\n") :
-		    strtok(config + 6 /* sizeof(struct sflap_hdr) */  + strlen("CONFIG:"), "\n");
+			strtok(config, "\n") :
+			strtok(config + 6 /* sizeof(struct sflap_hdr) */  + strlen("CONFIG:"), "\n");
 		do {
 			if (c == NULL)
 				break;
 			if (*c == 'g') {
 				strncpy(current, c + 2, sizeof(current));
-				add_group(gc, current);
-				how_many++;
+				if (!find_group(gc, current)) {
+					add_group(gc, current);
+					how_many++;
+				}
 			} else if (*c == 'b' && !find_buddy(gc, c + 2)) {
-				char nm[80], sw[80], *tmp = c + 2;
+				char nm[80], sw[388], *tmp = c + 2;
 				int i = 0;
-				while (*tmp != ':' && *tmp)
+				while (*tmp != ':' && *tmp && i < sizeof(nm) - 1)
 					nm[i++] = *tmp++;
+				
+				while (*tmp != ':' && *tmp)
+					tmp++;
+				
 				if (*tmp == ':')
 					*tmp++ = '\0';
+				
 				nm[i] = '\0';
 				i = 0;
-				while (*tmp)
+				while (*tmp && i < sizeof(sw) - 1)
 					sw[i++] = *tmp++;
 				sw[i] = '\0';
-				if (!find_buddy(gc, nm))
+				if (!find_buddy(gc, nm)) {
 					add_buddy(gc, current, nm, sw);
-				how_many++;
-
-				bud = g_list_append(bud, c + 2);
+					how_many++;
+					bud = g_list_append(bud, c + 2);
+				}
 			} else if (*c == 'p') {
 				GSList *d = gc->permit;
 				char *n;
@@ -379,13 +395,14 @@ void parse_toc_buddy_list(struct gaim_connection *gc, char *config, int from_do_
 				n = g_strdup(normalize (name));
 				while (d) {
 					if (!g_strcasecmp(n, normalize (d->data)))
-						 break;
+						break;
 					d = d->next;
 				}
 				g_free(n);
-				if (!d)
+				if (!d) {
 					gc->permit = g_slist_append(gc->permit, name);
-				else
+					how_many++;
+				} else
 					g_free(name);
 			} else if (*c == 'd') {
 				GSList *d = gc->deny;
@@ -395,14 +412,15 @@ void parse_toc_buddy_list(struct gaim_connection *gc, char *config, int from_do_
 				n = g_strdup(normalize (name));
 				while (d) {
 					if (!g_strcasecmp(n, normalize (d->data)))
-						 break;
-					d = d->next;
-				}
-				g_free(n);
-				if (!d)
-					gc->deny = g_slist_append(gc->deny, name);
-				else
-					g_free(name);
+					break;
+				d = d->next;
+			}
+			g_free(n);
+			if (!d) {
+				gc->deny = g_slist_append(gc->deny, name);
+				how_many++;
+			} else
+				g_free(name);
 			} else if (!strncmp("toc", c, 3)) {
 				sscanf(c + strlen(c) - 1, "%d", &gc->permdeny);
 				debug_printf("permdeny: %d\n", gc->permdeny);
@@ -415,22 +433,17 @@ void parse_toc_buddy_list(struct gaim_connection *gc, char *config, int from_do_
 					gc->permdeny = 1;
 			}
 		} while ((c = strtok(NULL, "\n")));
-
+		
 		if (bud != NULL) {
 			serv_add_buddies(gc, bud);
 			g_list_free(bud);
 		}
 		serv_set_permit_deny(gc);
 	}
-
-	/* perhaps the server dropped the buddy list, try importing from
-	   cache */
-
-	if (how_many == 0 && !from_do_import) {
-		do_import(gc, NULL);
-	} else if (gc && (bud_list_cache_exists(gc) == FALSE)) {
+	
+	if (how_many != 0)
 		do_export(gc);
-	}
+	
 }
 
 void toc_build_config(struct gaim_connection *gc, char *s, int len, gboolean show)
@@ -473,131 +486,158 @@ void toc_build_config(struct gaim_connection *gc, char *s, int len, gboolean sho
 	}
 }
 
-/* remove leading whitespace from a string */
-static char *remove_spaces(char *str)
-{
-	int i;
-	char *new;
-
-	if (str == NULL)
-		return NULL;
-
-	i = strspn(str, " \t\n\r\f");
-	new = &str[i];
-
-	return new;
-}
-
-
-/* translate an AIM 3 buddylist (*.lst) to a GAIM buddylist */
-static void translate_lst(FILE *src_fp, char *dest)
+/* translate an AIM 3 buddylist (*.lst) to a Gaim buddylist */
+static GString *translate_lst(FILE *src_fp)
 {
 	char line[BUF_LEN], *line2;
 	char *name;
 	int i;
 
-	sprintf(dest, "m 1\n");
+	GString *dest = g_string_new("m 1\n");
 
 	while (fgets(line, BUF_LEN, src_fp)) {
-		line2 = remove_spaces(line);
+		line2 = g_strchug(line);
 		if (strstr(line2, "group") == line2) {
 			name = strpbrk(line2, " \t\n\r\f") + 1;
-			strcat(dest, "g ");
+			dest = g_string_append(dest, "g ");
 			for (i = 0; i < strcspn(name, "\n\r"); i++)
 				if (name[i] != '\"')
-					strncat(dest, &name[i], 1);
-			strcat(dest, "\n");
+					dest = g_string_append_c(dest, name[i]);
+			dest = g_string_append_c(dest, '\n');
 		}
 		if (strstr(line2, "buddy") == line2) {
 			name = strpbrk(line2, " \t\n\r\f") + 1;
-			strcat(dest, "b ");
+			dest = g_string_append(dest, "b ");
 			for (i = 0; i < strcspn(name, "\n\r"); i++)
 				if (name[i] != '\"')
-					strncat(dest, &name[i], 1);
-			strcat(dest, "\n");
+					dest = g_string_append_c(dest, name[i]);
+			dest = g_string_append_c(dest, '\n');
 		}
 	}
 
-	return;
+	return dest;
 }
 
 
-/* translate an AIM 4 buddylist (*.blt) to GAIM format */
-static void translate_blt(FILE *src_fp, char *dest)
+/* translate an AIM 4 buddylist (*.blt) to Gaim format */
+static GString *translate_blt(FILE *src_fp)
 {
 	int i;
 	char line[BUF_LEN];
 	char *buddy;
 
-	sprintf(dest, "m 1\n");
+	GString *dest = g_string_new("m 1\n");
 
 	while (strstr(fgets(line, BUF_LEN, src_fp), "Buddy") == NULL);
 	while (strstr(fgets(line, BUF_LEN, src_fp), "list") == NULL);
 
 	while (1) {
-		fgets(line, BUF_LEN, src_fp);
+		fgets(line, BUF_LEN, src_fp); g_strchomp(line);
 		if (strchr(line, '}') != NULL)
 			break;
 
-		/* Syntax starting with "<group> {" */
 		if (strchr(line, '{') != NULL) {
-			strcat(dest, "g ");
-			buddy = remove_spaces(strtok(line, "{"));
-			for (i = 0; i < strlen(buddy); i++) {
+			/* Syntax starting with "<group> {" */
+
+			dest = g_string_append(dest, "g ");
+			buddy = g_strchug(strtok(line, "{"));
+			for (i = 0; i < strlen(buddy); i++)
 				if (buddy[i] != '\"')
-					strncat(dest, &buddy[i], 1);
-			}
-			strcat(dest, "\n");
+					dest = g_string_append_c(dest, buddy[i]);
+			dest = g_string_append_c(dest, '\n');
 			while (strchr(fgets(line, BUF_LEN, src_fp), '}') == NULL) {
-				buddy = remove_spaces(line);
-				strcat(dest, "b ");
-				if (strchr(buddy, '\"') != NULL) {
+				gboolean pounce = FALSE;
+				char *e;
+				g_strchomp(line);
+				buddy = g_strchug(line);
+				debug_printf("\nbuddy: \"%s\"\n\n", buddy);
+				dest = g_string_append(dest, "b ");
+				if (strchr(buddy, '{') != NULL) {
+					/* buddy pounce, etc */
+					char *pos = strchr(buddy, '{') - 1;
+					*pos = 0;
+					pounce = TRUE;
+				}
+				if ((e = strchr(buddy, '\"')) != NULL) {
+					*e = '\0';
 					buddy++;
-					strncat(dest, buddy, strchr(buddy, '\"') - buddy);
-					strcat(dest, "\n");
-				} else
-					strcat(dest, buddy);
+				}
+				dest = g_string_append(dest, buddy);
+				dest = g_string_append_c(dest, '\n');
+				if (pounce)
+					do
+						fgets(line, BUF_LEN, src_fp);
+					while (!strchr(line, '}'));
 			}
-		}
-		/* Syntax "group buddy buddy ..." */
-		else {
-			buddy = remove_spaces(strtok(line, " \n"));
-			strcat(dest, "g ");
+		} else {
+
+			/* Syntax "group buddy buddy ..." */
+			buddy = g_strchug(strtok(line, " \n"));
+			dest = g_string_append(dest, "g ");
 			if (strchr(buddy, '\"') != NULL) {
-				strcat(dest, &buddy[1]);
-				strcat(dest, " ");
-				buddy = remove_spaces(strtok(NULL, " \n"));
+				dest = g_string_append(dest, &buddy[1]);
+				dest = g_string_append_c(dest, ' ');
+				buddy = g_strchug(strtok(NULL, " \n"));
 				while (strchr(buddy, '\"') == NULL) {
-					strcat(dest, buddy);
-					strcat(dest, " ");
-					buddy = remove_spaces(strtok(NULL, " \n"));
+					dest = g_string_append(dest, buddy);
+					dest = g_string_append_c(dest, ' ');
+					buddy = g_strchug(strtok(NULL, " \n"));
 				}
-				strncat(dest, buddy, strlen(buddy) - 1);
+				buddy[strlen(buddy) - 1] = '\0';
+				dest = g_string_append(dest, buddy);
 			} else {
-				strcat(dest, buddy);
+				dest = g_string_append(dest, buddy);
 			}
-			strcat(dest, "\n");
-			while ((buddy = remove_spaces(strtok(NULL, " \n"))) != NULL) {
-				strcat(dest, "b ");
+			dest = g_string_append_c(dest, '\n');
+			while ((buddy = g_strchug(strtok(NULL, " \n"))) != NULL) {
+				dest = g_string_append(dest, "b ");
 				if (strchr(buddy, '\"') != NULL) {
-					strcat(dest, &buddy[1]);
-					strcat(dest, " ");
-					buddy = remove_spaces(strtok(NULL, " \n"));
+					dest = g_string_append(dest, &buddy[1]);
+					dest = g_string_append_c(dest, ' ');
+					buddy = g_strchug(strtok(NULL, " \n"));
 					while (strchr(buddy, '\"') == NULL) {
-						strcat(dest, buddy);
-						strcat(dest, " ");
-						buddy = remove_spaces(strtok(NULL, " \n"));
+						dest = g_string_append(dest, buddy);
+						dest = g_string_append_c(dest, ' ');
+						buddy = g_strchug(strtok(NULL, " \n"));
 					}
-					strncat(dest, buddy, strlen(buddy) - 1);
+					buddy[strlen(buddy) - 1] = '\0';
+					dest = g_string_append(dest, buddy);
 				} else {
-					strcat(dest, buddy);
+					dest = g_string_append(dest, buddy);
 				}
-				strcat(dest, "\n");
+				dest = g_string_append_c(dest, '\n');
 			}
 		}
 	}
 
-	return;
+	return dest;
+}
+
+static GString *translate_gnomeicu(FILE *src_fp)
+{
+	char line[BUF_LEN];
+	GString *dest = g_string_new("m 1\ng Buddies\n");
+
+	while (strstr(fgets(line, BUF_LEN, src_fp), "NewContacts") == NULL);
+
+	while (fgets(line, BUF_LEN, src_fp)) {
+		char *eq;
+		g_strchomp(line);
+		if (line[0] == '\n' || line[0] == '[')
+			break;
+		eq = strchr(line, '=');
+		if (!eq)
+			break;
+		*eq = ':';
+		eq = strchr(eq, ',');
+		if (eq)
+			*eq = '\0';
+		dest = g_string_append(dest, "b ");
+		dest = g_string_append(dest, line);
+		dest = g_string_append_c(dest, '\n');
+	}
+
+	return dest;
 }
 
 static gchar *get_screenname_filename(const char *name)
@@ -628,8 +668,7 @@ gboolean bud_list_cache_exists(struct gaim_connection *gc)
 
 	file = gaim_user_dir();
 	if (file != (char *)NULL) {
-		g_snprintf(path, sizeof path, "%s/%s.%d.blist", file, g_screenname,
-			   (gc->protocol == PROTO_OSCAR) ? PROTO_TOC : gc->protocol);
+		g_snprintf(path, sizeof path, "%s/%s.%d.blist", file, g_screenname, gc->protocol);
 		if (!stat(path, &sbuf)) {
 			debug_printf("%s exists.\n", path);
 			ret = TRUE;
@@ -653,86 +692,83 @@ gboolean bud_list_cache_exists(struct gaim_connection *gc)
 
 void do_import(struct gaim_connection *gc, char *filename)
 {
-	char *buf = g_malloc(BUF_LONG * 2);
-	char *buf2;
-	char *first = g_malloc(64);
-	char *file;
+	GString *buf = NULL;
+	char first[64];
 	char path[PATHSIZE];
-	char *g_screenname;
 	int len;
 	FILE *f;
+	struct stat st;
 
 	if (filename) {
 		g_snprintf(path, sizeof(path), "%s", filename);
 	} else {
-		g_screenname = get_screenname_filename(gc->username);
+		char *g_screenname = get_screenname_filename(gc->username);
+		char *file = gaim_user_dir();
 
-		file = gaim_user_dir();
 		if (file != (char *)NULL) {
-			sprintf(path, "%s/%s.%d.blist", file, g_screenname,
-				(gc->protocol == PROTO_OSCAR) ? PROTO_TOC : gc->protocol);
+			sprintf(path, "%s/%s.%d.blist", file, g_screenname, gc->protocol);
 			g_free(file);
 			g_free(g_screenname);
 		} else {
 			g_free(g_screenname);
-			g_free(buf);
-			g_free(first);
 			return;
 		}
 	}
 
+	if (stat(path, &st)) {
+		debug_printf("Unable to stat %s.\n", path);
+		return;
+	}
+
 	if (!(f = fopen(path, "r"))) {
 		debug_printf("Unable to open %s.\n", path);
-		g_free(buf);
-		g_free(first);
 		return;
 	}
 
 	fgets(first, 64, f);
 
-	/* AIM 4 buddy list */
+	if ((first[0] == '\n') || (first[0] == '\r' && first[1] == '\n'))
+		fgets(first, 64, f);
+
 	if (!g_strncasecmp(first, "Config {", strlen("Config {"))) {
+		/* AIM 4 buddy list */
 		debug_printf("aim 4\n");
 		rewind(f);
-		translate_blt(f, buf);
-		debug_printf("%s\n", buf);
-		buf2 = buf;
-		buf = g_malloc(8193);
-		g_snprintf(buf, 8192, "toc_set_config {%s}\n", buf2);
-		g_free(buf2);
-		/* AIM 3 buddy list */
+		buf = translate_blt(f);
 	} else if (strstr(first, "group") != NULL) {
+		/* AIM 3 buddy list */
 		debug_printf("aim 3\n");
 		rewind(f);
-		translate_lst(f, buf);
-		debug_printf("%s\n", buf);
-		buf2 = buf;
-		buf = g_malloc(8193);
-		g_snprintf(buf, 8192, "toc_set_config {%s}\n", buf2);
-		g_free(buf2);
-		/* GAIM buddy list - no translation */
-	} else if (first[0] == 'm') {
+		buf = translate_lst(f);
+	} else if (!g_strncasecmp(first, "[User]", strlen("[User]"))) {
+		/* GnomeICU (hopefully) */
+		debug_printf("gnomeicu\n");
 		rewind(f);
-		len = fread(buf, 1, BUF_LONG * 2, f);
-		buf[len] = '\0';
-		buf2 = buf;
-		buf = g_malloc(8193);
-		g_snprintf(buf, 8192, "toc_set_config {%s}\n", buf2);
-		g_free(buf2);
-		/* Something else */
-	} else {
-		g_free(buf);
-		g_free(first);
-		fclose(f);
-		return;
+		buf = translate_gnomeicu(f);
+	} else if (first[0] == 'm') {
+		/* Gaim buddy list - no translation */
+		char buf2[BUF_LONG * 2];
+		buf = g_string_new("");
+		rewind(f);
+		while (1) {
+			len = fread(buf2, 1, BUF_LONG * 2 - 1, f);
+			if (len <= 0)
+				break;
+			buf2[len] = '\0';
+			buf = g_string_append(buf, buf2);
+			if (len != BUF_LONG * 2 - 1)
+				break;
+		}
 	}
 
-	parse_toc_buddy_list(gc, buf, 1);
-
 	fclose(f);
 
-	g_free(buf);
-	g_free(first);
+	if (buf) {
+		buf = g_string_prepend(buf, "toc_set_config {");
+		buf = g_string_append(buf, "}\n");
+		parse_toc_buddy_list(gc, buf->str);
+		g_string_free(buf, TRUE);
+	}
 }
 
 void do_export(struct gaim_connection *g)
@@ -757,8 +793,7 @@ void do_export(struct gaim_connection *g)
 
 	g_screenname = get_screenname_filename(g->username);
 
-	sprintf(path, "%s/%s.%d.blist", file, g_screenname,
-		(g->protocol == PROTO_OSCAR) ? PROTO_TOC : g->protocol);
+	sprintf(path, "%s/%s.%d.blist", file, g_screenname, g->protocol);
 	if ((f = fopen(path, "w"))) {
 		debug_printf("writing %s\n", path);
 		toc_build_config(g, buf, 8192 - 1, TRUE);
@@ -772,3 +807,63 @@ void do_export(struct gaim_connection *g)
 	g_free(g_screenname);
 	g_free(file);
 }
+
+static gboolean is_blocked(struct buddy *b)
+{
+	struct gaim_connection *gc = b->gc;
+
+	if (gc->permdeny == PERMIT_ALL)
+		return FALSE;
+
+	if (gc->permdeny == PERMIT_NONE) {
+		if (g_strcasecmp(b->name, gc->displayname))
+			return TRUE;
+		else
+			return FALSE;
+	}
+
+	if (gc->permdeny == PERMIT_SOME) {
+		char *x = g_strdup(normalize(b->name));
+		GSList *s = gc->permit;
+		while (s) {
+			if (!g_strcasecmp(x, normalize(s->data)))
+				break;
+			s = s->next;
+		}
+		g_free(x);
+		if (s)
+			return FALSE;
+		return TRUE;
+	}
+
+	if (gc->permdeny == DENY_SOME) {
+		char *x = g_strdup(normalize(b->name));
+		GSList *s = gc->deny;
+		while (s) {
+			if (!g_strcasecmp(x, normalize(s->data)))
+				break;
+			s = s->next;
+		}
+		g_free(x);
+		if (s)
+			return TRUE;
+		return FALSE;
+	}
+
+	return FALSE;
+}
+
+void signoff_blocked(struct gaim_connection *gc)
+{
+	GSList *g = gc->groups;
+	while (g) {
+		GSList *m = ((struct group *)g->data)->members;
+		while (m) {
+			struct buddy *b = m->data;
+			if (is_blocked(b))
+				serv_got_update(gc, b->name, 0, 0, 0, 0, 0, 0);
+			m = m->next;
+		}
+		g = g->next;
+	}
+}
