@@ -50,6 +50,7 @@ struct debug_window *dw = NULL;
 static GtkWidget *prefs = NULL;
 
 static GtkWidget *gaim_button(const char *, guint *, int, GtkWidget *);
+static GtkWidget *blist_tab_radio(const char *, int, GtkWidget *, GtkWidget *);
 static void prefs_build_general();
 static void prefs_build_buddy();
 static void prefs_build_convo();
@@ -183,6 +184,7 @@ static void general_page()
 	GtkWidget *sep;
 	GtkWidget *idle;
 	GtkWidget *opt;
+	GtkWidget *typingbutton;
 
 	parent = prefdialog->parent;
 	gtk_widget_destroy(prefdialog);
@@ -226,7 +228,15 @@ static void general_page()
 		misc_options ^= OPT_MISC_DEBUG;
 	debugbutton = gaim_button(_("Show Debug Window"), &misc_options, OPT_MISC_DEBUG, mbox);
 	gtk_signal_connect(GTK_OBJECT(debugbutton), "destroy", GTK_SIGNAL_FUNC(destdeb), 0);
+	
+	/* Preferences should be positive */
+	typingbutton = gaim_button(_("Notify buddies that you are typing to them"), &misc_options,
+				   OPT_MISC_STEALTH_TYPING, mbox);
 
+	/* So we have to toggle it */
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(typingbutton), !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(typingbutton)));
+	misc_options ^= OPT_MISC_STEALTH_TYPING;
+	
 	frame = gtk_frame_new(_("Report Idle Times"));
 	gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 5);
 	gtk_widget_show(frame);
@@ -236,7 +246,7 @@ static void general_page()
 	gtk_widget_show(mbox);
 
 	idle = idle_radio(_("None"), IDLE_NONE, mbox, NULL);
-	idle = idle_radio(_("GAIM Use"), IDLE_GAIM, mbox, idle);
+	idle = idle_radio(_("Gaim Use"), IDLE_GAIM, mbox, idle);
 #ifdef USE_SCREENSAVER
 	idle = idle_radio(_("X Use"), IDLE_SCREENSAVER, mbox, idle);
 #endif
@@ -279,6 +289,7 @@ static void general_page()
 	gtk_widget_show(vbox);
 
 	opt = browser_radio(_("KFM"), BROWSER_KFM, vbox, NULL);
+	opt = browser_radio(_("Opera"), BROWSER_OPERA, vbox, opt);
 	opt = browser_radio(_("Netscape"), BROWSER_NETSCAPE, vbox, opt);
 
 	new_window =
@@ -291,6 +302,7 @@ static void general_page()
 #ifdef USE_GNOME
 	opt = browser_radio(_("GNOME URL Handler"), BROWSER_GNOME, vbox, opt);
 #endif /* USE_GNOME */
+	opt = browser_radio(_("Galeon"), BROWSER_GALEON, vbox, opt);
 	opt = browser_radio(_("Manual"), BROWSER_MANUAL, vbox, opt);
 
 	browser_entry = gtk_entry_new();
@@ -338,9 +350,9 @@ static void proxy_print_option(GtkEntry *entry, int entrynum)
 static void proxy_print_optionrad(GtkRadioButton * entry, int entrynum)
 {
 	if (entrynum == PROXY_NONE)
-		gtk_widget_set_sensitive(GTK_WIDGET(prefs_proxy_frame), FALSE);
+		gtk_widget_set_sensitive(prefs_proxy_frame, FALSE);
 	else
-		gtk_widget_set_sensitive(GTK_WIDGET(prefs_proxy_frame), TRUE);
+		gtk_widget_set_sensitive(prefs_proxy_frame, TRUE);
 
 	proxytype = entrynum;
 	save_prefs();
@@ -389,6 +401,9 @@ static void proxy_page()
 	gtk_widget_show(table);
 	gtk_container_add(GTK_CONTAINER(frame), table);
 
+	frame = gtk_frame_new(_("Proxy Server"));
+	prefs_proxy_frame = frame;
+
 	first = gtk_radio_button_new_with_label(NULL, _("No Proxy"));
 	gtk_table_attach(GTK_TABLE(table), first, 0, 1, 0, 1, GTK_FILL | GTK_EXPAND, 0, 0, 0);
 
@@ -429,13 +444,10 @@ static void proxy_page()
 		gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(opt), TRUE);
 
 
-	frame = gtk_frame_new(_("Proxy Server"));
 	gtk_container_set_border_width(GTK_CONTAINER(frame), 5);
 	gtk_widget_show(frame);
 	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 5);
 
-	prefs_proxy_frame = frame;
-
 	if (proxytype == PROXY_NONE)
 		gtk_widget_set_sensitive(GTK_WIDGET(frame), FALSE);
 
@@ -520,6 +532,8 @@ static void buddy_page()
 	GtkWidget *frame;
 	GtkWidget *hbox;
 	GtkWidget *vbox;
+	GtkWidget *sep;
+	GtkWidget *opt;
 	GtkWidget *button;
 	GtkWidget *button2;
 
@@ -542,11 +556,29 @@ static void buddy_page()
 	gtk_box_pack_start(GTK_BOX(box), frame, FALSE, FALSE, 5);
 	gtk_widget_show(frame);
 
-	hbox = gtk_hbox_new(TRUE, 5);
+	hbox = gtk_hbox_new(FALSE, 5);
 	gtk_container_add(GTK_CONTAINER(frame), hbox);
 	gtk_widget_show(hbox);
 
-	vbox = gtk_vbox_new(FALSE, 5);
+	/* "Place blist tabs  */
+	vbox = gtk_vbox_new(TRUE, 5);
+	gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 5);
+	gtk_widget_show(vbox);
+
+	label = gtk_label_new(_("Tab Placement:"));
+	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 5);
+	gtk_widget_show(label);	
+	
+	opt = blist_tab_radio(_("Top"), ~(OPT_BLIST_BOTTOM_TAB), vbox, NULL);
+	opt = blist_tab_radio(_("Bottom"), OPT_BLIST_BOTTOM_TAB, vbox, opt);
+
+	sep = gtk_vseparator_new();
+	gtk_box_pack_start(GTK_BOX(hbox), sep, FALSE, FALSE, 5);
+	gtk_widget_show(sep);
+
+	/* End of blist tab options */
+
+	vbox = gtk_vbox_new(TRUE, 5);
 	gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 5);
 	gtk_widget_show(vbox);
 
@@ -554,21 +586,16 @@ static void buddy_page()
 #ifdef USE_APPLET
 	gaim_button(_("Automatically show buddy list on sign on"), &blist_options,
 		    OPT_BLIST_APP_BUDDY_SHOW, vbox);
+	gaim_button(_("Display Buddy List near applet"), &blist_options, OPT_BLIST_NEAR_APPLET, vbox);
+	
 #endif
 	gaim_button(_("Save Window Size/Position"), &blist_options, OPT_BLIST_SAVED_WINDOWS, vbox);
 
-	vbox = gtk_vbox_new(FALSE, 5);
-	gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 5);
-	gtk_widget_show(vbox);
-
 	button2 =
 	    gaim_button(_("Show pictures on buttons"), &blist_options, OPT_BLIST_SHOW_BUTTON_XPM, vbox);
 	if (blist_options & OPT_BLIST_NO_BUTTONS)
 		gtk_widget_set_sensitive(button2, FALSE);
 	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(toggle_sensitive), button2);
-#ifdef USE_APPLET
-	gaim_button(_("Display Buddy List near applet"), &blist_options, OPT_BLIST_NEAR_APPLET, vbox);
-#endif
 
 	frame = gtk_frame_new(_("Group Displays"));
 	gtk_box_pack_start(GTK_BOX(box), frame, FALSE, FALSE, 5);
@@ -610,6 +637,7 @@ static void buddy_page()
 	gtk_widget_show(vbox);
 
 	gaim_button(_("Show idle times"), &blist_options, OPT_BLIST_SHOW_IDLETIME, vbox);
+	gaim_button(_("Grey idle buddies"), &blist_options, OPT_BLIST_GREY_IDLERS, vbox);
 
 	gtk_widget_show(prefdialog);
 }
@@ -763,16 +791,30 @@ static GtkWidget *am_radio(char *label, int which, GtkWidget *box, GtkWidget *se
 static void set_tab_opt(GtkWidget *w, int data)
 {
 	int mask;
-	if (data & 0x1) {	/* set the first bit if we're affecting chat buttons */
+	if (convo_options & OPT_CONVO_COMBINE) {
+		/* through an amazing coincidence (this wasn't planned), we're able to do this,
+		 * since the two sets of options end up having the same value. isn't that great. */
 		mask = (OPT_CHAT_SIDE_TAB | OPT_CHAT_BR_TAB);
 		chat_options &= ~(mask);
 		chat_options |= (data & mask);
-		update_chat_tabs();
-	} else {
+
 		mask = (OPT_IM_SIDE_TAB | OPT_IM_BR_TAB);
 		im_options &= ~(mask);
 		im_options |= (data & mask);
+
 		update_im_tabs();
+	} else {
+		if (data & 0x1) {	/* set the first bit if we're affecting chat buttons */
+			mask = (OPT_CHAT_SIDE_TAB | OPT_CHAT_BR_TAB);
+			chat_options &= ~(mask);
+			chat_options |= (data & mask);
+			update_chat_tabs();
+		} else {
+			mask = (OPT_IM_SIDE_TAB | OPT_IM_BR_TAB);
+			im_options &= ~(mask);
+			im_options |= (data & mask);
+			update_im_tabs();
+		}
 	}
 
 	save_prefs();
@@ -812,7 +854,7 @@ static void update_spin_value(GtkWidget *w, GtkWidget *spin)
 	*value = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin));
 }
 
-static void gaim_labeled_spin_button(GtkWidget *box, const gchar *title, int *val)
+static void gaim_labeled_spin_button(GtkWidget *box, const gchar *title, int *val, int min, int max)
 {
 	GtkWidget *hbox;
 	GtkWidget *label;
@@ -827,7 +869,7 @@ static void gaim_labeled_spin_button(GtkWidget *box, const gchar *title, int *va
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
 	gtk_widget_show(label);
 
-	adjust = gtk_adjustment_new(*val, 1, 9999, 1, 1, 1);
+	adjust = gtk_adjustment_new(*val, min, max, 1, 1, 1);
 	spin = gtk_spin_button_new(GTK_ADJUSTMENT(adjust), 1, 0);
 	gtk_object_set_user_data(GTK_OBJECT(spin), val);
 	gtk_widget_set_usize(spin, 50, -1);
@@ -837,6 +879,13 @@ static void gaim_labeled_spin_button(GtkWidget *box, const gchar *title, int *va
 	gtk_widget_show(spin);
 }
 
+static gboolean current_is_im = FALSE;
+
+static void not_im()
+{
+	current_is_im = FALSE;
+}
+
 static void im_page()
 {
 	GtkWidget *parent;
@@ -855,8 +904,11 @@ static void im_page()
 	parent = prefdialog->parent;
 	gtk_widget_destroy(prefdialog);
 
+	current_is_im = TRUE;
+
 	prefdialog = gtk_frame_new(_("IM Options"));
 	gtk_container_add(GTK_CONTAINER(parent), prefdialog);
+	gtk_signal_connect(GTK_OBJECT(prefdialog), "destroy", GTK_SIGNAL_FUNC(not_im), NULL);
 
 	box = gtk_vbox_new(FALSE, 5);
 	gtk_container_set_border_width(GTK_CONTAINER(box), 5);
@@ -902,8 +954,17 @@ static void im_page()
 	button =
 	    gaim_button(_("Show all conversations in one tabbed window"), &im_options, OPT_IM_ONE_WINDOW,
 			vbox2);
+	opt = gaim_button(_("Show chats in the same tabbed window"), &convo_options, OPT_CONVO_COMBINE, vbox2);
+	if (chat_options & OPT_CHAT_ONE_WINDOW) {
+		if (!(im_options & OPT_IM_ONE_WINDOW))
+			gtk_widget_set_sensitive(opt, FALSE);
+		gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(toggle_sensitive), opt);
+	} else
+		gtk_widget_set_sensitive(opt, FALSE);
 	gaim_button(_("Raise windows on events"), &im_options, OPT_IM_POPUP, vbox2);
 	gaim_button(_("Show logins in window"), &im_options, OPT_IM_LOGON, vbox2);
+	gaim_button(_("Show aliases in tabs/titles"), &im_options, OPT_IM_ALIAS_TAB, vbox2);
+	gaim_button(_("Hide window on send"), &im_options, OPT_IM_POPDOWN, vbox2);
 
 	frame = gtk_frame_new(_("Window Sizes"));
 	gtk_box_pack_start(GTK_BOX(box), frame, FALSE, FALSE, 5);
@@ -913,9 +974,9 @@ static void im_page()
 	gtk_container_add(GTK_CONTAINER(frame), vbox);
 	gtk_widget_show(vbox);
 
-	gaim_labeled_spin_button(vbox, _("New window width:"), &conv_size.width);
-	gaim_labeled_spin_button(vbox, _("New window height:"), &conv_size.height);
-	gaim_labeled_spin_button(vbox, _("Entry widget height:"), &conv_size.entry_height);
+	gaim_labeled_spin_button(vbox, _("New window width:"), &conv_size.width, 25, 9999);
+	gaim_labeled_spin_button(vbox, _("New window height:"), &conv_size.height, 25, 9999);
+	gaim_labeled_spin_button(vbox, _("Entry widget height:"), &conv_size.entry_height, 25, 9999);
 
 	frame = gtk_frame_new(_("Tab Placement"));
 	gtk_box_pack_start(GTK_BOX(box), frame, FALSE, FALSE, 5);
@@ -965,6 +1026,12 @@ static void im_page()
 	gtk_widget_show(vbox);
 
 	gaim_button(_("Hide Buddy Icons"), &im_options, OPT_IM_HIDE_ICONS, vbox);
+
+	vbox = gtk_vbox_new(FALSE, 5);
+	gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 5);
+	gtk_widget_show(vbox);
+	
+	gaim_button(_("Disable Buddy Icon Animation"), &im_options, OPT_IM_NO_ANIMATION, vbox);
 #endif
 
 	gtk_widget_show(prefdialog);
@@ -1039,6 +1106,13 @@ static void chat_page()
 	button =
 	    gaim_button(_("Show all chats in one tabbed window"), &chat_options, OPT_CHAT_ONE_WINDOW,
 			vbox2);
+	opt = gaim_button(_("Show conversations in the same tabbed window"), &convo_options, OPT_CONVO_COMBINE, vbox2);
+	if (im_options & OPT_IM_ONE_WINDOW) {
+		if (!(chat_options & OPT_CHAT_ONE_WINDOW))
+			gtk_widget_set_sensitive(opt, FALSE);
+		gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(toggle_sensitive), opt);
+	} else
+		gtk_widget_set_sensitive(opt, FALSE);
 	gaim_button(_("Raise windows on events"), &chat_options, OPT_CHAT_POPUP, vbox2);
 	gaim_button(_("Show people joining/leaving in window"), &chat_options, OPT_CHAT_LOGON, vbox2);
 
@@ -1050,9 +1124,9 @@ static void chat_page()
 	gtk_container_add(GTK_CONTAINER(frame), vbox);
 	gtk_widget_show(vbox);
 
-	gaim_labeled_spin_button(vbox, _("New window width:"), &buddy_chat_size.width);
-	gaim_labeled_spin_button(vbox, _("New window height:"), &buddy_chat_size.height);
-	gaim_labeled_spin_button(vbox, _("Entry widget height:"), &buddy_chat_size.entry_height);
+	gaim_labeled_spin_button(vbox, _("New window width:"), &buddy_chat_size.width, 25, 9999);
+	gaim_labeled_spin_button(vbox, _("New window height:"), &buddy_chat_size.height, 25, 9999);
+	gaim_labeled_spin_button(vbox, _("Entry widget height:"), &buddy_chat_size.entry_height, 25, 9999);
 
 	frame = gtk_frame_new(_("Tab Placement"));
 	gtk_box_pack_start(GTK_BOX(box), frame, FALSE, FALSE, 5);
@@ -1356,6 +1430,7 @@ static void font_page()
 
 static GtkWidget *sndent[NUM_SOUNDS];
 static GtkWidget *sndcmd = NULL;
+static char *last_sound_dir = NULL;
 
 void close_sounddialog(GtkWidget *w, GtkWidget *w2)
 {
@@ -1396,6 +1471,21 @@ void do_select_sound(GtkWidget *w, int snd)
 
 	/* Close the window! It's getting cold in here! */
 	close_sounddialog(NULL, sounddialog);
+
+	if (last_sound_dir)
+		g_free(last_sound_dir);
+	last_sound_dir = g_dirname(sound_file[snd]);
+}
+
+static void test_sound(GtkWidget *button, int snd)
+{
+	guint32 tmp_sound = sound_options;
+	if (!(sound_options & OPT_SOUND_WHEN_AWAY))
+		sound_options ^= OPT_SOUND_WHEN_AWAY;
+	if (!(sound_options & sounds[snd].opt))
+		sound_options ^= sounds[snd].opt;
+	play_sound(snd);
+	sound_options = tmp_sound;
 }
 
 static void reset_sound(GtkWidget *button, int snd)
@@ -1409,7 +1499,6 @@ static void reset_sound(GtkWidget *button, int snd)
 
 static void sel_sound(GtkWidget *button, int snd)
 {
-
 	char *buf = g_malloc(BUF_LEN);
 
 	if (!sounddialog) {
@@ -1417,7 +1506,7 @@ static void sel_sound(GtkWidget *button, int snd)
 
 		gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(sounddialog));
 
-		g_snprintf(buf, BUF_LEN - 1, "%s/", getenv("HOME"));
+		g_snprintf(buf, BUF_LEN - 1, "%s/", last_sound_dir ? last_sound_dir : g_get_home_dir());
 
 		gtk_file_selection_set_filename(GTK_FILE_SELECTION(sounddialog), buf);
 
@@ -1436,7 +1525,7 @@ static void sel_sound(GtkWidget *button, int snd)
 	gdk_window_raise(sounddialog->window);
 }
 
-static void sound_entry(char *label, int opt, GtkWidget *box, int snd)
+static void sound_entry(GtkWidget *box, int snd)
 {
 	GtkWidget *hbox;
 	GtkWidget *entry;
@@ -1446,15 +1535,20 @@ static void sound_entry(char *label, int opt, GtkWidget *box, int snd)
 	gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, FALSE, 0);
 	gtk_widget_show(hbox);
 
-	gaim_button(label, &sound_options, opt, hbox);
+	gaim_button(sounds[snd].label, &sound_options, sounds[snd].opt, hbox);
+
+	button = gtk_button_new_with_label(_("Test"));
+	gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 3);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(test_sound), (void *)snd);
+	gtk_widget_show(button);
 
 	button = gtk_button_new_with_label(_("Reset"));
-	gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 5);
+	gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 3);
 	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(reset_sound), (void *)snd);
 	gtk_widget_show(button);
 
 	button = gtk_button_new_with_label(_("Choose..."));
-	gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 0);
+	gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 3);
 	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(sel_sound), (void *)snd);
 	gtk_widget_show(button);
 
@@ -1478,6 +1572,20 @@ static gint sound_cmd_yeah(GtkEntry *entry, GdkEvent *event, gpointer d)
 	return TRUE;
 }
 
+static void set_sound_driver(GtkWidget *w, int option)
+{
+	if (option == OPT_SOUND_CMD)
+		gtk_widget_set_sensitive(sndcmd, TRUE);
+	else
+		gtk_widget_set_sensitive(sndcmd, FALSE);
+
+	sound_options &= ~(OPT_SOUND_NORMAL | OPT_SOUND_BEEP |
+			   OPT_SOUND_NAS | OPT_SOUND_ARTSC |
+			   OPT_SOUND_ESD | OPT_SOUND_CMD);
+	sound_options |= option;
+	save_prefs();
+}
+
 static void sound_page()
 {
 	GtkWidget *parent;
@@ -1488,6 +1596,10 @@ static void sound_page()
 	GtkWidget *hbox;
 	GtkWidget *vbox2;
 	GtkWidget *sep;
+	GtkWidget *omenu;
+	GtkWidget *menu;
+	GtkWidget *opt;
+	int i=1, driver=0, j;
 
 	parent = prefdialog->parent;
 	gtk_widget_destroy(prefdialog);
@@ -1521,13 +1633,12 @@ static void sound_page()
 	gtk_widget_show(vbox2);
 
 	gaim_button(_("No sounds when you log in"), &sound_options, OPT_SOUND_SILENT_SIGNON, vbox2);
-	gaim_button(_("Sounds while away"), &sound_options, OPT_SOUND_WHEN_AWAY, vbox2);
 
 	vbox2 = gtk_vbox_new(FALSE, 5);
 	gtk_box_pack_start(GTK_BOX(hbox), vbox2, TRUE, TRUE, 5);
 	gtk_widget_show(vbox2);
 
-	gaim_button(_("Beep instead of playing sound"), &sound_options, OPT_SOUND_BEEP, vbox2);
+	gaim_button(_("Sounds while away"), &sound_options, OPT_SOUND_WHEN_AWAY, vbox2);
 
 	sep = gtk_hseparator_new();
 	gtk_box_pack_start(GTK_BOX(vbox), sep, FALSE, FALSE, 0);
@@ -1537,7 +1648,81 @@ static void sound_page()
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
 	gtk_widget_show(hbox);
 
-	label = gtk_label_new(_("Command to play sound files\n(%s for filename; internal if empty)"));
+	label = gtk_label_new(_("Sound method"));
+	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
+	gtk_widget_show(label);
+
+	omenu = gtk_option_menu_new();
+	menu = gtk_menu_new();
+
+	opt = gtk_menu_item_new_with_label("Console Beep");
+	gtk_signal_connect(GTK_OBJECT(opt), "activate",
+			   GTK_SIGNAL_FUNC(set_sound_driver), 
+			   (gpointer)OPT_SOUND_BEEP);
+	gtk_widget_show(opt);
+	gtk_menu_append(GTK_MENU(menu), opt);
+	if ((sound_options & OPT_SOUND_BEEP) && !driver) driver = i;
+	i++;
+
+#ifdef ESD_SOUND
+	opt = gtk_menu_item_new_with_label("ESD");
+	gtk_signal_connect(GTK_OBJECT(opt), "activate",
+			   GTK_SIGNAL_FUNC(set_sound_driver), 
+			   (gpointer)OPT_SOUND_ESD);
+	gtk_widget_show(opt);
+	gtk_menu_append(GTK_MENU(menu), opt);
+	if ((sound_options & OPT_SOUND_ESD) && !driver) driver = i;
+	i++;
+#endif
+#ifdef ARTSC_SOUND
+	opt = gtk_menu_item_new_with_label("ArtsC");
+	gtk_signal_connect(GTK_OBJECT(opt), "activate",
+			   GTK_SIGNAL_FUNC(set_sound_driver), 
+			   (gpointer)OPT_SOUND_ARTSC);
+	gtk_widget_show(opt);
+	gtk_menu_append(GTK_MENU(menu), opt);
+	if ((sound_options & OPT_SOUND_ARTSC) && !driver) driver = i;
+	i++;
+#endif
+#ifdef NAS_SOUND
+	opt = gtk_menu_item_new_with_label("NAS");
+	gtk_signal_connect(GTK_OBJECT(opt), "activate",
+			   GTK_SIGNAL_FUNC(set_sound_driver), 
+			   (gpointer)OPT_SOUND_NAS);
+	gtk_widget_show(opt);
+	gtk_menu_append(GTK_MENU(menu), opt);
+	if ((sound_options & OPT_SOUND_NAS) && !driver) driver = i;
+	i++;
+#endif
+
+	opt = gtk_menu_item_new_with_label("Internal");
+	gtk_signal_connect(GTK_OBJECT(opt), "activate",
+			   GTK_SIGNAL_FUNC(set_sound_driver), 
+			   (gpointer)OPT_SOUND_NORMAL);
+	gtk_widget_show(opt);
+	gtk_menu_append(GTK_MENU(menu), opt);
+	if ((sound_options & OPT_SOUND_NORMAL) && !driver) driver = i;
+	i++;
+
+	opt = gtk_menu_item_new_with_label("Command");
+	gtk_signal_connect(GTK_OBJECT(opt), "activate",
+			   GTK_SIGNAL_FUNC(set_sound_driver), 
+			   (gpointer)OPT_SOUND_CMD);
+	gtk_widget_show(opt);
+	gtk_menu_append(GTK_MENU(menu), opt);
+	if ((sound_options & OPT_SOUND_CMD) && !driver) driver = i;
+	i++;
+
+	gtk_option_menu_set_menu(GTK_OPTION_MENU(omenu), menu);
+	gtk_option_menu_set_history(GTK_OPTION_MENU(omenu), driver - 1);
+	gtk_box_pack_start(GTK_BOX(hbox), omenu, FALSE, FALSE, 5);
+	gtk_widget_show_all(omenu);
+
+	hbox = gtk_hbox_new(TRUE, 5);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
+	gtk_widget_show(hbox);
+
+	label = gtk_label_new(_("Sound command\n(%s for filename)"));
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
 	gtk_widget_show(label);
 
@@ -1546,9 +1731,10 @@ static void sound_page()
 	gtk_entry_set_text(GTK_ENTRY(sndcmd), sound_cmd);
 	gtk_box_pack_end(GTK_BOX(hbox), sndcmd, FALSE, FALSE, 5);
 	gtk_signal_connect(GTK_OBJECT(sndcmd), "focus_out_event", GTK_SIGNAL_FUNC(sound_cmd_yeah), NULL);
+	gtk_widget_set_sensitive(sndcmd, (OPT_SOUND_CMD & sound_options));
 	gtk_widget_show(sndcmd);
 
-	frame = gtk_frame_new(_("Events"));
+	frame = gtk_frame_new(_("Sound played when:"));
 	gtk_box_pack_start(GTK_BOX(box), frame, FALSE, FALSE, 5);
 	gtk_widget_show(frame);
 
@@ -1556,26 +1742,20 @@ static void sound_page()
 	gtk_container_add(GTK_CONTAINER(frame), vbox);
 	gtk_widget_show(vbox);
 
-	sound_entry(_("Sound when buddy logs in"), OPT_SOUND_LOGIN, vbox, BUDDY_ARRIVE);
-	sound_entry(_("Sound when buddy logs out"), OPT_SOUND_LOGOUT, vbox, BUDDY_LEAVE);
+	for (j=0; j < NUM_SOUNDS; j++) {
+		/* no entry for sounds without an option */
+		if (sounds[j].opt == 0)
+			continue;
 
-	sep = gtk_hseparator_new();
-	gtk_box_pack_start(GTK_BOX(vbox), sep, FALSE, FALSE, 5);
-	gtk_widget_show(sep);
+		/* seperators before SND_RECEIVE and SND_CHAT_JOIN */
+		if ((j == SND_RECEIVE) || (j == SND_CHAT_JOIN)) {
+			sep = gtk_hseparator_new();
+			gtk_box_pack_start(GTK_BOX(vbox), sep, FALSE, FALSE, 5);
+			gtk_widget_show(sep);
+		}
 
-	sound_entry(_("Sound when received message begins conversation"), OPT_SOUND_FIRST_RCV, vbox,
-		    FIRST_RECEIVE);
-	sound_entry(_("Sound when message is received"), OPT_SOUND_RECV, vbox, RECEIVE);
-	sound_entry(_("Sound when message is sent"), OPT_SOUND_SEND, vbox, SEND);
-
-	sep = gtk_hseparator_new();
-	gtk_box_pack_start(GTK_BOX(vbox), sep, FALSE, FALSE, 5);
-	gtk_widget_show(sep);
-
-	sound_entry(_("Sound in chat rooms when people enter"), OPT_SOUND_CHAT_JOIN, vbox, CHAT_JOIN);
-	sound_entry(_("Sound in chat rooms when people leave"), OPT_SOUND_CHAT_PART, vbox, CHAT_LEAVE);
-	sound_entry(_("Sound in chat rooms when you talk"), OPT_SOUND_CHAT_YOU_SAY, vbox, CHAT_YOU_SAY);
-	sound_entry(_("Sound in chat rooms when others talk"), OPT_SOUND_CHAT_SAY, vbox, CHAT_SAY);
+		sound_entry(vbox, j);
+	}
 
 	gtk_widget_show(prefdialog);
 }
@@ -1597,9 +1777,9 @@ void away_list_clicked(GtkWidget *widget, struct away_message *a)
 	/* Fill the text box with new message */
 	strcpy(buffer, a->message);
 	tmp = stylize(buffer, BUF_LONG);
-	gtk_imhtml_append_text(GTK_IMHTML(away_text), tmp, GTK_IMHTML_NO_TITLE |
+	gtk_imhtml_append_text(GTK_IMHTML(away_text), tmp, -1, GTK_IMHTML_NO_TITLE |
 			       GTK_IMHTML_NO_COMMENTS | GTK_IMHTML_NO_SCROLL);
-	gtk_imhtml_append_text(GTK_IMHTML(away_text), "<BR>", GTK_IMHTML_NO_TITLE |
+	gtk_imhtml_append_text(GTK_IMHTML(away_text), "<BR>", -1, GTK_IMHTML_NO_TITLE |
 			       GTK_IMHTML_NO_COMMENTS | GTK_IMHTML_NO_SCROLL);
 	g_free(tmp);
 }
@@ -1624,6 +1804,7 @@ static void paldest(GtkWidget *m, gpointer n)
 {
 	gtk_widget_destroy(prefs_away_list);
 	prefs_away_list = NULL;
+	prefs_away_menu = NULL;
 	make_away_button = NULL;
 }
 
@@ -1672,8 +1853,6 @@ void default_away_menu_init(GtkWidget *omenu)
 	gtk_option_menu_set_history(GTK_OPTION_MENU(omenu), g_slist_index(away_messages, default_away));
 }
 
-
-
 static void away_page()
 {
 	GtkWidget *parent;
@@ -1683,11 +1862,12 @@ static void away_page()
 	GtkWidget *vbox;
 	GtkWidget *hbox;
 	GtkWidget *vbox2;
+	GtkWidget *button;
+	GtkWidget *button2;
 	GtkWidget *top;
 	GtkWidget *bot;
 	GtkWidget *sw;
 	GtkWidget *sw2;
-	GtkWidget *button;
 	GtkWidget *list_item;
 	GtkWidget *sep;
 	GtkObject *adjust;
@@ -1735,7 +1915,12 @@ static void away_page()
 	gtk_box_pack_start(GTK_BOX(hbox), vbox2, TRUE, TRUE, 5);
 	gtk_widget_show(vbox2);
 
-	gaim_button(_("Don't send auto-response"), &away_options, OPT_AWAY_NO_AUTO_RESP, vbox2);
+	button = gaim_button(_("Don't send auto-response"), &away_options, OPT_AWAY_NO_AUTO_RESP, vbox2);
+	button2 = gaim_button(_("Only send auto-response when idle"), &away_options, OPT_AWAY_IDLE_RESP,
+			      vbox2);
+	if (away_options & OPT_AWAY_NO_AUTO_RESP)
+		gtk_widget_set_sensitive(button2, FALSE);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(toggle_sensitive), button2);
 	gaim_button(_("Queue new messages when away"), &away_options, OPT_AWAY_QUEUE, vbox2);
 
 	sep = gtk_hseparator_new();
@@ -1746,6 +1931,21 @@ static void away_page()
 	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 	gtk_widget_show(hbox);
 
+	gaim_labeled_spin_button(hbox, _("Time between sending auto-responses (in seconds):"),
+			&away_resend, 1, 24 * 60 * 60);
+
+	if (away_options & OPT_AWAY_NO_AUTO_RESP)
+		gtk_widget_set_sensitive(hbox, FALSE);
+	gtk_signal_connect(GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(toggle_sensitive), hbox);
+
+	sep = gtk_hseparator_new();
+	gtk_box_pack_start(GTK_BOX(vbox), sep, FALSE, FALSE, 0);
+	gtk_widget_show(sep);
+
+	hbox = gtk_hbox_new(FALSE, 5);
+	gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
+	gtk_widget_show(hbox);
+
 	button = gaim_button(_("Auto Away after"), &away_options, OPT_AWAY_AUTO, hbox);
 
 	adjust = gtk_adjustment_new(auto_away, 1, 1440, 1, 10, 10);
@@ -1817,6 +2017,8 @@ static void away_page()
 
 	away_text = gtk_imhtml_new(NULL, NULL);
 	gtk_container_add(GTK_CONTAINER(sw2), away_text);
+	GTK_LAYOUT(away_text)->hadjustment->step_increment = 10.0;
+	GTK_LAYOUT(away_text)->vadjustment->step_increment = 10.0;
 	gaim_setup_imhtml(away_text);
 	gtk_widget_show(away_text);
 
@@ -1848,9 +2050,9 @@ static void away_page()
 		a = (struct away_message *)awy->data;
 		g_snprintf(buffer, sizeof(buffer), "%s", a->message);
 		tmp = stylize(buffer, BUF_LONG);
-		gtk_imhtml_append_text(GTK_IMHTML(away_text), tmp, GTK_IMHTML_NO_TITLE |
+		gtk_imhtml_append_text(GTK_IMHTML(away_text), tmp, -1, GTK_IMHTML_NO_TITLE |
 				       GTK_IMHTML_NO_COMMENTS | GTK_IMHTML_NO_SCROLL);
-		gtk_imhtml_append_text(GTK_IMHTML(away_text), "<BR>", GTK_IMHTML_NO_TITLE |
+		gtk_imhtml_append_text(GTK_IMHTML(away_text), "<BR>", -1, GTK_IMHTML_NO_TITLE |
 				       GTK_IMHTML_NO_COMMENTS | GTK_IMHTML_NO_SCROLL);
 		g_free(tmp);
 	}
@@ -2031,7 +2233,7 @@ static void build_deny_menu()
 		c = c->next;
 		if (!gc->prpl->set_permit_deny)
 			continue;
-		g_snprintf(buf, sizeof buf, "%s (%s)", gc->username, (*gc->prpl->name)());
+		g_snprintf(buf, sizeof buf, "%s (%s)", gc->username, gc->prpl->name());
 		opt = gtk_menu_item_new_with_label(buf);
 		gtk_signal_connect(GTK_OBJECT(opt), "activate", GTK_SIGNAL_FUNC(deny_gc_opt), gc);
 		gtk_widget_show(opt);
@@ -2255,14 +2457,14 @@ static void try_me(GtkCTree *ctree, GtkCTreeNode *node)
 	/* this is a hack */
 	void (*func)();
 	func = gtk_ctree_node_get_row_data(ctree, node);
-	(*func)();
+	func();
 }
 
 void show_prefs()
 {
 	GtkWidget *vbox;
 	GtkWidget *hpaned;
-	GtkWidget *scroll;
+	/* GtkWidget *scroll; */
 	GtkWidget *container;
 	GtkWidget *hbox;
 	GtkWidget *close;
@@ -2289,18 +2491,22 @@ void show_prefs()
 	gtk_box_pack_start(GTK_BOX(vbox), hpaned, TRUE, TRUE, 0);
 	gtk_widget_show(hpaned);
 
+	/*
 	scroll = gtk_scrolled_window_new(NULL, NULL);
 	gtk_paned_pack1(GTK_PANED(hpaned), scroll, FALSE, FALSE);
 	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll), GTK_POLICY_NEVER, GTK_POLICY_NEVER);
 	gtk_widget_set_usize(scroll, 125, -1);
 	gtk_widget_show(scroll);
+	*/
 
 	preftree = gtk_ctree_new(1, 0);
 	gtk_ctree_set_line_style(GTK_CTREE(preftree), GTK_CTREE_LINES_SOLID);
 	gtk_ctree_set_expander_style(GTK_CTREE(preftree), GTK_CTREE_EXPANDER_TRIANGLE);
 	gtk_clist_set_reorderable(GTK_CLIST(preftree), FALSE);
-	gtk_container_add(GTK_CONTAINER(scroll), preftree);
+	/* gtk_container_add(GTK_CONTAINER(scroll), preftree); */
+	gtk_paned_pack1(GTK_PANED(hpaned), preftree, FALSE, FALSE);
 	gtk_signal_connect(GTK_OBJECT(preftree), "tree_select_row", GTK_SIGNAL_FUNC(try_me), NULL);
+	gtk_widget_set_usize(preftree, 125, -1);
 	gtk_widget_show(preftree);
 
 	container = gtk_frame_new(NULL);
@@ -2354,10 +2560,12 @@ static void build_debug()
 	if (!dw)
 		dw = g_new0(struct debug_window, 1);
 
-	dw->window = gtk_window_new(GTK_WINDOW_DIALOG);
-	gtk_window_set_title(GTK_WINDOW(dw->window), _("GAIM debug output window"));
+	GAIM_DIALOG(dw->window);
+	gtk_window_set_title(GTK_WINDOW(dw->window), _("Gaim debug output window"));
 	gtk_window_set_wmclass(GTK_WINDOW(dw->window), "debug_out", "Gaim");
 	gtk_signal_connect(GTK_OBJECT(dw->window), "delete_event", GTK_SIGNAL_FUNC(debug_delete), NULL);
+	gtk_widget_realize(dw->window);
+	aol_icon(dw->window->window);
 
 	box = gtk_hbox_new(FALSE, 0);
 	gtk_container_add(GTK_CONTAINER(dw->window), box);
@@ -2411,9 +2619,8 @@ void debug_printf(char *fmt, ...)
 		if (scroll)
 			gtk_adjustment_set_value(adj, adj->upper - adj->lower - adj->page_size);
 	}
-#ifdef DEBUG
-	g_print("%s", s);
-#endif
+	if (opt_debug)
+		g_print("%s", s);
 	g_free(s);
 }
 
@@ -2481,10 +2688,8 @@ static void set_blist_option(GtkWidget *w, int option)
 	if (!blist)
 		return;
 
-	if (option == OPT_BLIST_NO_BUTTONS) {
+	if (option == OPT_BLIST_NO_BUTTONS)
 		build_imchat_box(!(blist_options & OPT_BLIST_NO_BUTTONS));
-		update_button_pix();
-	}
 
 	if (option == OPT_BLIST_SHOW_GRPNUM)
 		update_num_groups();
@@ -2492,11 +2697,15 @@ static void set_blist_option(GtkWidget *w, int option)
 	if (option == OPT_BLIST_NO_MT_GRP)
 		toggle_show_empty_groups();
 
-	if (option == OPT_BLIST_SHOW_BUTTON_XPM)
+	if ((option == OPT_BLIST_SHOW_BUTTON_XPM) || (option == OPT_BLIST_NO_BUTTONS))
 		update_button_pix();
 
 	if (option == OPT_BLIST_SHOW_PIXMAPS)
 		toggle_buddy_pixmaps();
+
+	if ((option == OPT_BLIST_GREY_IDLERS) || (option == OPT_BLIST_SHOW_IDLETIME))
+		update_idle_times();
+
 }
 
 static void set_convo_option(GtkWidget *w, int option)
@@ -2512,6 +2721,20 @@ static void set_convo_option(GtkWidget *w, int option)
 	if (option == OPT_CONVO_CHECK_SPELLING)
 		toggle_spellchk();
 
+	if (option == OPT_CONVO_COMBINE) {
+		/* (OPT_IM_SIDE_TAB | OPT_IM_BR_TAB) == (OPT_CHAT_SIDE_TAB | OPT_CHAT_BR_TAB) */
+		if (current_is_im) {
+			int set = im_options & (OPT_IM_SIDE_TAB | OPT_IM_BR_TAB);
+			chat_options &= ~(OPT_CHAT_SIDE_TAB | OPT_CHAT_BR_TAB);
+			chat_options |= set;
+		} else {
+			int set = chat_options & (OPT_IM_SIDE_TAB | OPT_IM_BR_TAB);
+			im_options &= ~(OPT_CHAT_SIDE_TAB | OPT_CHAT_BR_TAB);
+			im_options |= set;
+		}
+		convo_tabize();
+	}
+
 	save_prefs();
 }
 
@@ -2520,11 +2743,17 @@ static void set_im_option(GtkWidget *w, int option)
 	im_options ^= option;
 
 	if (option == OPT_IM_ONE_WINDOW)
-		tabize();
+		im_tabize();
 
 	if (option == OPT_IM_HIDE_ICONS)
 		set_hide_icons();
 
+	if (option == OPT_IM_ALIAS_TAB)
+		set_convo_titles();
+
+	if (option == OPT_IM_NO_ANIMATION)
+		set_anim();
+
 	save_prefs();
 }
 
@@ -2604,6 +2833,38 @@ GtkWidget *gaim_button(const char *text, guint *options, int option, GtkWidget *
 	return button;
 }
 
+static void blist_tab_opt(GtkWidget *widget, int option)
+{
+	/* Following general method of set_tab_opt()  */
+	int mask;
+	mask = (OPT_BLIST_BOTTOM_TAB);
+	
+	blist_options &= ~(mask);
+	blist_options |= (mask & option);	
+	
+	set_blist_tab();
+}
+
+static GtkWidget *blist_tab_radio(const char *label, int which, GtkWidget *box,GtkWidget *set) 
+{
+        GtkWidget *new_opt;
+
+        if(!set)
+                new_opt = gtk_radio_button_new_with_label (NULL, label);        
+	else
+                new_opt = gtk_radio_button_new_with_label(gtk_radio_button_group(GTK_RADIO_BUTTON(set)), label); 
+        
+	gtk_box_pack_start(GTK_BOX(box), new_opt, FALSE, FALSE, 0);
+        gtk_signal_connect(GTK_OBJECT(new_opt), "clicked", 
+			GTK_SIGNAL_FUNC(blist_tab_opt), (void *)which);
+	gtk_widget_show(new_opt);
+
+        if ((blist_options & OPT_BLIST_BOTTOM_TAB) == (which & OPT_BLIST_BOTTOM_TAB))
+                gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(new_opt), TRUE);
+
+	return new_opt;        
+}
+
 void prefs_build_general()
 {
 	GtkCTreeNode *node;
