@@ -47,6 +47,7 @@
 #include "pixmaps/away_icon.xpm"
 #include "pixmaps/dt_icon.xpm"
 #include "pixmaps/free_icon.xpm"
+#include "pixmaps/wireless_icon.xpm"
 
 #define REVISION "penguin"
 
@@ -83,6 +84,7 @@
 #define UC_ADMIN	0x04
 #define UC_UNCONFIRMED	0x08
 #define UC_NORMAL	0x10
+#define UC_WIRELESS     0x20
 
 struct ft_request {
 	struct gaim_connection *gc;
@@ -469,7 +471,7 @@ static char *show_error_message()
                 g_snprintf(buf, sizeof(buf), _("An unknown signon error has occurred: %s."), w);
                 break;
         default:
-                g_snprintf(buf, sizeof(buf), _("An unknown error, %d, has occured.  Info: %s"), no, w);
+                g_snprintf(buf, sizeof(buf), _("An unknown error, %d, has occurred.  Info: %s"), no, w);
 	}
 
         return buf;
@@ -486,7 +488,7 @@ static void toc_callback(gpointer data, gint source, GaimInputCondition conditio
 
 	/* there's data waiting to be read, so read it. */
 	if (wait_reply(gc, buf, 8 * 1024) <= 0) {
-		hide_login_progress(gc, _("Connection Closed"));
+		hide_login_progress_error(gc, _("Connection Closed"));
 		signoff(gc);
 		return;
 	}
@@ -645,6 +647,9 @@ static void toc_callback(gpointer data, gint source, GaimInputCondition conditio
 		case 'O':
 			type |= UC_NORMAL;
 			break;
+		case 'C':
+			type |= UC_WIRELESS;
+			break;
 		default:
 			break;
 		}
@@ -774,7 +779,7 @@ static void toc_callback(gpointer data, gint source, GaimInputCondition conditio
 	} else if (!strcasecmp(c, "DIR_STATUS")) {
 	} else if (!strcasecmp(c, "ADMIN_NICK_STATUS")) {
 	} else if (!strcasecmp(c, "ADMIN_PASSWD_STATUS")) {
-		do_error_dialog(_("Password Change Successeful"), _("Gaim - Password Change"));
+		do_error_dialog(_("Password Change Successful"), _("Gaim - Password Change"));
 	} else if (!strcasecmp(c, "PAUSE")) {
 		tdt->state = STATE_PAUSE;
 		do_error_dialog(_("TOC has sent a PAUSE command. When this happens, TOC ignores"
@@ -1189,6 +1194,8 @@ static char **toc_list_icon(int uc)
 		return (char **)admin_icon_xpm;
 	if (uc & UC_UNCONFIRMED)
 		return (char **)dt_icon_xpm;
+	if (uc & UC_WIRELESS)
+		return (char **)wireless_icon_xpm;
 	return NULL;
 }
 
